/*
 * spi.c for miniD pre OLED
 *
 * Low-level SPI funtionality.
 *
 * Initial version: 20/12/2017
 * Author: Tomi Nihtila, nihtila.com
 */ 

#include "spi.h"
#include <avr/io.h>
#include <avr/interrupt.h>



// ********************************************************
// INITIALISE SPI
//
void SPIInit(void) {
	
	// All CS, MOSI, and CLK out, MISO in, all default high
	DDRB |= 0xB6;
	PORTB = 0xF6;
	
	// Enable SPI
	SPCR0 = (0x50 | SPI_CLOCK);
}


// ********************************************************
// SPI START
// Control chip select signal.
//
void SPIStart(spiCS cs) {
	// chip select low
	switch(cs) {
		case SPI_CS_PGA:
			PORTB &= ~0x10;
			break;
		case SPI_CS_SRC:
			PORTB &= ~0x02;
			break;
		case SPI_CS_DAC:
			PORTB &= ~0x04;
			break;
	}
}


// ********************************************************
// SPI END
// Control chip select signal.
//
void SPIEnd(spiCS cs) {
	// chip select high
	switch(cs) {
		case SPI_CS_PGA:
			PORTB |= 0x10;
			break;
		case SPI_CS_SRC:
			PORTB |= 0x02;
			break;
		case SPI_CS_DAC:
			PORTB |= 0x04;
			break;
	}
}


// ********************************************************
// SPI TRANSFER BYTE
// As SPI is shift register, this function is used both writing and reading byte.
//
unsigned char SPITransferByte(unsigned char data) {
	
	unsigned char temp;

	// disable interrupts
	cli();
	
	// Send byte
	SPDR0 = data;
	// Ensure transfer is complete; dummy implementation as it does not have any timeout
	while(!(SPSR0 & 0x80));
	// Read value shifted in
	temp = SPDR0;

	// enable interrupts
	sei();

	return temp;
}
